#include <Wire.h>
#include <L3G.h>

L3G gyro;

int sampleNum=500;
int dc_offset=0;
double noise=0;
unsigned long time;
int sampleTime=10;
int rate;
int prev_rate=0;
double angle=0;

void setup() {
  Serial.begin(115200);
  Wire.begin();

  while (!gyro.init());

  gyro.enableDefault();
  
  //Calculate initial DC offset and noise level of gyro
  
  for(int n=0;n<sampleNum;n++){
    gyro.read();
    dc_offset+=(int)gyro.g.z;
  }
  dc_offset=dc_offset/sampleNum;
  
  for(int n=0;n<sampleNum;n++){
    gyro.read();
    if((int)gyro.g.z-dc_offset>noise)
      noise=(int)gyro.g.z-dc_offset;
    else if((int)gyro.g.z-dc_offset<-noise)
      noise=-(int)gyro.g.z-dc_offset;
  }
  noise=noise/100; //gyro returns hundredths of degrees/sec
  
  //print dc offset and noise level
  Serial.println();
  Serial.print("DC Offset: ");
  Serial.print(dc_offset);
  Serial.print("\tNoise Level: ");
  Serial.println(noise);
  delay(7000);
}

void loop() {
  // Every 10 ms take a sample from the gyro
  if(millis() - time > sampleTime)
  {
    time = millis();  // update the time to get the next sample
    gyro.read();
    rate=((int)gyro.g.z-dc_offset)/100; //gyro returns hundredths of degrees/sec
    
    // Integrate the angular veloity to obtain angular position (or inclination)
    // Using the trapeziod method for numerical integration.
    //          (previous_Rate + current_Rate) * sampleTime
    // Angle = ---------------------------------------------- 
    //                        2 * 1000                            
    // because the sampleTime is expressed in milliseconds and rate in degree 
    // per seconds we also need to divide by 1000 ms/s to obtain the correct units.
    
    // Ignore the gyro if our angular velocity does not meet our threshold
    // if(rate >= noise || rate <= -noise) 
      angle += ((double)(prev_rate + rate) * sampleTime) / 2000;
      
    // remember the current speed for the next loop rate integration.
    prev_rate = rate;
    
    // Keep our angle between 0-359 degrees
    if (angle < 0)
      angle += 360;
    else if (angle >= 360)
      angle -= 360;
    
    //Code to correct for gyro drift when rover is motionless
    //dc_offset = (sampleNum–1)*dc_offset+(1/sampleNum)*(int)gyro.g.z;
    
    
    Serial.print("angle: ");
    Serial.print(angle);
    Serial.print("\trate: ");
    Serial.println(rate);
  }
}
